from __future__ import print_function

import argparse
import json
import os
import sys

from jsonschema import Draft7Validator
from jsonschema import validate as schema_validate
from jsonschema.exceptions import ValidationError


def _parse_cmdline_args():
    """
    Command line argument parsing. Doing it here means less duplication than
    would be the case in bin/

    Returns command line arguments as a dict
    """
    parser = argparse.ArgumentParser(description="Validate marugoto game content.")

    parser.add_argument(
        "-nw",
        "--no-warnings",
        default=True,
        action="store_true",
        required=False,
        help="Do not show warnings",
    )

    parser.add_argument(
        "-f",
        "--fail-first",
        default=False,
        action="store_true",
        required=False,
        help="Stop after first error",
    )

    parser.add_argument("path", help="Path to the files")

    return vars(parser.parse_args())


def _get_json_files(path):
    """
    Recursively get JSON files.
    """
    path = os.path.expanduser(path)
    print("Validating content at {}".format(path))
    out = []
    for root, _, files in os.walk(path):
        for file in files:
            if file.endswith(".json"):
                out.append(os.path.join(root, file))
    if not out:
        raise OSError("No .json found at {}.".format(os.path.abspath(path)))
    else:
        print("{} JSON files found.".format(len(out)))
    return out


def _get_schemata():
    """
    Load all our JSON schemata into a dict with file basenames as keys
    """
    schemata = dict()
    # hopfully, find the schemata folder?
    data_dir = os.path.join(os.path.dirname(os.path.dirname(__file__)), "schemata")
    jsons = [i for i in os.listdir(data_dir) if i.endswith(".json")]
    for jso in jsons:
        with open(os.path.join(data_dir, jso), "r") as fo:
            loaded = json.load(fo)
            schemata[os.path.splitext(jso)[0]] = loaded
    return schemata


def _print_errors(errors):
    """
    Nicely print out errors
    """
    eq = "=" * 100
    mi = "-" * 100
    if errors:
        print(eq)
    for i, (err, filename, invalid_syntax) in enumerate(errors, start=1):
        errname = "Content" if not invalid_syntax else "Syntax"
        form = "Problem #{} -- {} error in {}\n{}\n\n{}\n\n{}"
        print(form.format(i, errname, filename, mi, err, eq))


def _get_correct_schema(json_file, schemata):
    """
    Get correct validation schema based on filename
    """
    no_ext = os.path.basename(os.path.splitext(json_file)[0])
    schema_name = "".join([i for i in no_ext if i.isalpha()])
    Draft7Validator.check_schema(schemata[schema_name])
    return schemata[schema_name]


def _locate_schemata():
    """
    schemata dir seems to move around depending on how you install!?
    """
    fpath = os.path.dirname(__file__)
    first = os.path.dirname(fpath)
    second = os.path.dirname(first)
    third = sys.prefix
    fourth = os.path.join(third, "maruvar")
    dirs = [first, second, third, fourth]
    for path in dirs:
        if os.path.isdir(os.path.join(path, "schemata")):
            return os.path.join(path, "schemata")
    raise ValueError("No schemata found in: {}".format(dirs))


def validate(path=None, fail_first=False, no_warnings=False):
    """
    Main validator routine

    Iterate over all JSON files, check that they can be loaded, and then run jsonschema.
    Handle fail fast option both during loading and validating.
    """
    errors = list()
    to_check = _get_json_files(path)
    schemata = _get_schemata()
    for json_file in to_check:
        schema = _get_correct_schema(json_file, schemata)
        try:
            with open(json_file, "r") as f:
                data = json.load(f)
        except json.JSONDecodeError as err:
            errors.append((err, json_file, True))
            if fail_first:
                break
            continue
        try:
            schema_validate(instance=data, schema=schema)
        except ValidationError as err:
            errors.append((err, json_file, False))
            if fail_first:
                break
    _print_errors(errors)


if __name__ == "__main__":
    validate(**_parse_cmdline_args())



from __future__ import print_function

import argparse
import os

JSON_TYPES = {
    "audiocomponent",
    "chapter",
    "character",
    "checkboxexercise",
    "dateexercise",
    "dialog",
    "dialogresponse",
    "dialogspeech",
    "imagecomponent",
    "linkcomponent",
    "mail",
    "notebookentry",
    "page",
    "pagetransition",
    "radiobuttonexercise",
    "textcomponent",
    "textexercise",
    "topic",
    "uploadexercise",
    "videocomponent",
}


def listed(s):
    got = [i.strip().lower() for i in s.split(",")]
    if not got:
        raise ValueError(
            "Please provide one or more JSON file types, separated by comma"
        )
    return got


def _parse_cmdline_args():
    """
    Command line argument parsing. Doing it here means less duplication than
    would be the case in bin/

    Returns command line arguments as a dict
    """
    desc = "Find directories containing all of the specified file types"
    parser = argparse.ArgumentParser(description=desc)

    parser.add_argument(
        "-not",
        "--inverse",
        default=False,
        action="store_true",
        required=False,
        help="Invert the matching criteria",
    )

    parser.add_argument(
        "-any",
        "--any-of",
        default=False,
        action="store_true",
        required=False,
        help="Get any, not all directories containing these files",
    )

    parser.add_argument(
        "-only",
        "--only",
        default=False,
        action="store_true",
        required=False,
        help="Get directories containing only these files (any/all will be ignored)",
    )

    parser.add_argument(
        "json_types", type=listed, help="Comma-separated list of JSON file types"
    )

    parser.add_argument("path", help="Path to the data directory")

    return vars(parser.parse_args())


def _explain(inverse, any_of, only, json_types, path):
    """
    Print a plain-language explanation of the search
    """
    any_all = "any of" if any_of else "all of"
    if only:
        any_all = "only"
    do_not = "NOT " if inverse else ""
    types = "\n\t*".join(sorted(json_types))
    exp = "Finding dirs in {} where {} the following files are {}present:\n\t*{}\n"
    print(exp.format(path, any_all, do_not, types))


def _check_folder(folder, inverse, any_of, only, json_types):
    files = [i for i in os.listdir(folder) if i.endswith(".json")]
    files = [os.path.splitext(i)[0].lower() for i in files]
    files = ["".join(x for x in i if x.isalpha()) for i in files]
    if only:
        match = all(i in json_types for i in files)
    else:
        matcher = any if any_of else all
        match = matcher(f in files for f in json_types)
    return match if not inverse else not match


def finder(inverse=False, any_of=False, only=False, json_types=None, path=None):
    eq = "=" * 80
    mi = "-" * 80
    print(eq)
    path = os.path.expanduser(path)
    _explain(inverse, any_of, only, json_types, path)
    print(mi)
    for root, dirs, _ in os.walk(path):
        for folder in sorted(dirs):
            folder = os.path.join(root, folder)
            if _check_folder(folder, inverse, any_of, only, json_types):
                print("Matching directory: {}".format(os.path.abspath(folder)))
    print(eq)


if __name__ == "__main__":
    finder(**_parse_cmdline_args())



import os

from setuptools import setup


def read(fname):
    """
    Helper to read README
    """
    return open(os.path.join(os.path.dirname(__file__), fname)).read().strip()


data_files = [
    "schemata/{}".format(i)
    for i in os.listdir("schemata")
    if os.path.isfile("schemata/{}".format(i))
]

setup(
    name="maruval",
    version="0.1.0",  # bump2version will edit this automatically!
    description="Validator for marugoto game content",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    url="http://github.com/uzh/maruval",
    author="Danny McDonald",
    include_package_data=True,
    zip_safe=False,
    packages=["maruval"],
    package_data={"maruval": data_files},
    data_files=[("schemata", data_files)],
    scripts=["bin/maruval", "bin/marufind", "bin/marupretty"],
    author_email="daniel.mcdonald@uzh.ch",
    license="MIT",
    keywords=[],
    install_requires=["jsonschema==3.0.1"],
    dependency_links=[],
)
